@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.2.0
+;; Version: 2.0.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 
 ;;; Code:
 
-(require 'phpinspect-class)
 (require 'phpinspect-parser)
 (require 'phpinspect-bmap)
 (require 'phpinspect-edtrack)
@@ -32,6 +31,7 @@
 (require 'phpinspect-resolvecontext)
 (require 'phpinspect-resolve)
 (require 'phpinspect-util)
+(require 'phpinspect-typedef)
 
 (phpinspect--declare-log-group 'buffer)
 
@@ -54,6 +54,8 @@ emacs buffer."
   (-last-indexed-bmap nil)
   (imports nil
            :type phpinspect-toc)
+  (used-traits nil
+               :type phpinspect-toc)
   (namespaces nil
               :type phpinspect-toc)
   (classes nil
@@ -158,25 +160,27 @@ linked with."
   (cond ((phpinspect-class-p token)
          (when-let ((class (gethash token (phpinspect-buffer-token-index buffer))))
            (remhash token (phpinspect-buffer-token-index buffer))
-           (phpinspect-project-delete-class (phpinspect-buffer-project buffer) class)))
+           (phpinspect-project-delete-typedef (phpinspect-buffer-project buffer) class)))
         ((or (phpinspect-const-p token) (phpinspect-variable-p token))
          (when-let ((var (gethash token (phpinspect-buffer-token-index buffer))))
            (remhash token (phpinspect-buffer-token-index buffer))
-           (when-let ((class (phpinspect-project-get-class
+           (when-let ((class (phpinspect-project-get-typedef
                               (phpinspect-buffer-project buffer)
                               (car var))))
-             (phpinspect--class-delete-variable class (cdr var)))))
+             (phpi-typedef-delete-property class (cdr var)))))
         ((phpinspect-function-p token)
          (when-let ((func (gethash token (phpinspect-buffer-token-index buffer))))
            (remhash token (phpinspect-buffer-token-index buffer))
            (cond ((phpinspect-project-p (car func))
                   (phpinspect-project-delete-function (phpinspect-buffer-project buffer) (phpinspect--function-name-symbol (cdr func))))
                  ((phpinspect--type-p (car func))
-                  (when-let ((class (phpinspect-project-get-class
+                  (when-let ((class (phpinspect-project-get-typedef
                                      (phpinspect-buffer-project buffer)
                                      (car func))))
-                    (phpinspect--class-delete-method class (cdr func))))
-                 (t (error "Invalid index location")))))
+                    (phpi-typedef-delete-method class (cdr func))))
+                 (t (phpinspect-message "Invalid index location, reindexing buffer")
+                    (phpinspect-buffer-reindex buffer)
+                    (error "invalid index location")))))
         (t (error "Cannot delete index for token %s" token))))
 
 (cl-defmethod phpinspect-buffer-namespace-at-point ((buffer phpinspect-buffer) (point integer))
@@ -185,6 +189,50 @@ linked with."
                     point)))
     (and namespace (phpinspect-meta-overlaps-point namespace point) namespace)))
 
+(defun phpinspect-buffer-get-type-resolver-for-class (buffer class-token)
+  (pcase-let* ((`(,imports ,namespace-name)
+                (phpinspect-get-token-index-context
+                 (phpinspect-buffer-namespaces buffer)
+                 (phpinspect-buffer-imports buffer)
+                 class-token)))
+    (phpinspect--make-type-resolver
+     imports
+     (phpinspect-class-block (phpinspect-meta-token class-token))
+     namespace-name)))
+
+(cl-defmethod phpinspect-buffer-index-used-traits ((buffer phpinspect-buffer) (uses (head phpinspect-splayt)))
+  (let ((update t))
+    (if (phpinspect-buffer-used-traits buffer)
+        (pcase-let ((`(,new ,deleted)
+                     (phpinspect-toc-update
+                      (phpinspect-buffer-used-traits buffer) uses (phpinspect-buffer-root-meta buffer))))
+          (unless (or new deleted)
+            ;; Nothing changed, don't update
+            (setq update nil)))
+      (setf (phpinspect-buffer-used-traits buffer) (phpinspect-make-toc uses)))
+
+    (when update
+      (let ((class-toc (phpinspect-buffer-classes buffer)))
+        (phpinspect-splayt-traverse-lr (class (phpinspect-toc-tree class-toc))
+          (let ((config (phpinspect-buffer-get-trait-configuration-between-points
+                         buffer (phpinspect-meta-start class) (phpinspect-meta-end class)
+                         (phpinspect-buffer-get-type-resolver-for-class buffer class))))
+
+            (when-let ((typedef (phpinspect-buffer-get-index-for-token buffer (phpinspect-meta-token class))))
+              (let ((new-extensions (seq-uniq (append
+                                               (phpi-typedef-set-trait-config typedef config)
+                                               (phpi-typedef-subscribed-to-types typedef))
+                                              #'phpinspect--type=)))
+                (phpi-typedef-update-extensions typedef new-extensions)))))))))
+
+(defun phpinspect-buffer-get-trait-configuration-between-points (buffer start end type-resolver)
+  (let ((uses (phpinspect-toc-tokens-in-region (phpinspect-buffer-used-traits buffer) start end))
+        config)
+    (dolist (use uses)
+      (setq config (nconc config (phpinspect--index-trait-use (phpinspect-meta-token use) type-resolver nil))))
+
+    config))
+
 (cl-defmethod phpinspect-buffer-index-imports ((buffer phpinspect-buffer) (imports (head phpinspect-splayt)))
   (let (to-be-indexed)
     (if (phpinspect-buffer-imports buffer)
@@ -224,6 +272,14 @@ linked with."
 
     (list imports namespace-name)))
 
+(defun phpinspect--buffer-update-type-declaration (buffer typedef declaration class-token imports namespace-name)
+  (phpi-typedef-update-declaration
+   typedef declaration imports namespace-name
+   (phpinspect-buffer-get-trait-configuration-between-points
+    buffer (phpinspect-meta-start class-token) (phpinspect-meta-end class-token)
+    (phpinspect--make-type-resolver
+     imports (phpinspect-class-block (phpinspect-meta-token class-token)) namespace-name))))
+
 (cl-defmethod phpinspect-buffer-index-classes ((buffer phpinspect-buffer) (classes (head phpinspect-splayt)))
   (let ((declarations (phpinspect-buffer-declarations buffer))
         (namespaces (phpinspect-buffer-namespaces buffer))
@@ -234,25 +290,29 @@ linked with."
                                                         (phpinspect-buffer-classes buffer)
                                                         classes (phpinspect-buffer-root-meta buffer)))
                      (new-declarations) (declaration) (replaced) (indexed) (class))
+          ;; Collect declarations of new classes
           (dolist (class new-classes)
             (when (setq declaration (phpinspect-toc-token-at-or-after-point declarations (phpinspect-meta-start class)))
               (push (cons (phpinspect-meta-token declaration) class) new-declarations)))
 
+          ;; Delete no longer existing classes from the index
           (dolist (deleted deleted-classes)
             (if (and (setq class (phpinspect-buffer-get-index-for-token
                                   buffer (phpinspect-meta-token deleted)))
-                     (setq replaced (assoc (phpinspect--class-declaration class) new-declarations #'equal)))
+                     (setq replaced (assoc (phpi-typedef-declaration class) new-declarations #'equal)))
                 (pcase-let ((`(,imports ,namespace-name) (phpinspect-get-token-index-context namespaces buffer-imports (cdr replaced))))
                   (phpinspect-buffer-update-index-reference-for-token
                    buffer (phpinspect-meta-token deleted) (phpinspect-meta-token (cdr replaced)))
-                  (phpinspect--class-update-declaration class (car replaced) imports namespace-name)
+                  (phpinspect--buffer-update-type-declaration
+                   buffer class (car replaced) (cdr replaced) imports namespace-name)
                   (push (cdr replaced) indexed))
               (phpinspect-buffer-delete-index-for-token buffer (phpinspect-meta-token deleted))))
 
           (dolist (class new-declarations)
             (unless (memq (cdr class) indexed)
               (let (imports namespace-name class-name class-obj)
-                (pcase-setq `(,imports ,namespace-name) (phpinspect-get-token-index-context namespaces buffer-imports (cdr class))
+                (pcase-setq `(,imports ,namespace-name)
+                            (phpinspect-get-token-index-context namespaces buffer-imports (cdr class))
                             `(,class-name) (phpinspect--index-class-declaration
                                             (car class)
                                             (phpinspect--make-type-resolver
@@ -260,9 +320,10 @@ linked with."
                                              (phpinspect-class-block (phpinspect-meta-token (cdr class)))
                                              namespace-name)))
                 (when class-name
-                  (setq  class-obj (phpinspect-project-get-class-create project class-name 'no-enqueue))
+                  (setq  class-obj (phpinspect-project-get-typedef-create project class-name 'no-enqueue))
                   (phpinspect-buffer-set-index-reference-for-token buffer (phpinspect-meta-token (cdr class)) class-obj)
-                  (phpinspect--class-update-declaration class-obj (car class) imports namespace-name))))))
+                  (phpinspect--buffer-update-type-declaration
+                   buffer class-obj (car class) (cdr class) imports namespace-name))))))
       ;; Else: Index all classes
       (setf (phpinspect-buffer-classes buffer) (phpinspect-make-toc classes))
       (phpinspect-splayt-traverse (class classes)
@@ -276,9 +337,10 @@ linked with."
                                        namespace-name)))
                      (class-obj))
           (when class-name
-            (setq class-obj (phpinspect-project-get-class-create project class-name 'no-enqueue))
+            (setq class-obj (phpinspect-project-get-typedef-create project class-name 'no-enqueue))
             (phpinspect-buffer-set-index-reference-for-token buffer (phpinspect-meta-token class) class-obj)
-            (phpinspect--class-update-declaration class-obj (phpinspect-meta-token declaration) imports namespace-name)))))))
+            (phpinspect--buffer-update-type-declaration
+             buffer class-obj (phpinspect-meta-token declaration) class imports namespace-name)))))))
 
 (cl-defmethod phpinspect-buffer-index-functions ((buffer phpinspect-buffer) (functions (head phpinspect-splayt)))
   (let ((classes (phpinspect-buffer-classes buffer))
@@ -331,14 +393,14 @@ linked with."
                              type-resolver
                              scope
                              (and (phpinspect-comment-p comment-before) comment-before)))
-              (unless (phpinspect--function-anonyous-p indexed)
+              (unless (phpinspect--function-anonymous-p indexed)
                 (if static
-                    (phpinspect--class-set-static-method class-obj indexed)
-                  (phpinspect--class-set-method class-obj indexed)))
+                    (phpi-typedef-set-static-method class-obj indexed)
+                  (phpi-typedef-set-method class-obj indexed)))
 
               (phpinspect-buffer-set-index-reference-for-token
                buffer (phpinspect-meta-token func)
-               (cons (phpinspect--class-name class-obj) indexed))))
+               (cons (phpi-typedef-name class-obj) indexed))))
         ;; Else: index function
         (pcase-let ((`(,imports ,namespace-name) (phpinspect-get-token-index-context namespaces imports func))
                     (comment-before (phpinspect-meta-find-left-sibling func)))
@@ -432,15 +494,15 @@ linked with."
 
           (when (and (phpinspect-variable-p (phpinspect-meta-token var)) (not (phpinspect--variable-type indexed)))
             (setf (phpinspect--variable-type indexed)
-                  (phpinspect--class-resolve-property-type
+                  (phpi-typedef-resolve-property-type
                    class-obj (phpinspect-buffer-project buffer)
                    (cadr (phpinspect-meta-token var)) type-resolver class)))
 
-          (phpinspect--class-set-variable class-obj indexed)
+          (phpi-typedef-set-property class-obj indexed)
 
           (phpinspect-buffer-set-index-reference-for-token
            buffer (phpinspect-meta-token var)
-           (cons (phpinspect--class-name class-obj) indexed)))))))
+           (cons (phpi-typedef-name class-obj) indexed)))))))
 
 (cl-defmethod phpinspect-buffer-reset ((buffer phpinspect-buffer))
   (setf (phpinspect-buffer-tree buffer) nil)
@@ -485,6 +547,7 @@ continuing execution."
       (let ((map (phpinspect-buffer-map buffer)))
         (unless (eq map (phpinspect-buffer--last-indexed-bmap buffer))
           (phpinspect--log "Updating project index")
+          (phpinspect-buffer-index-used-traits buffer (phpinspect-bmap-used-traits map))
           (phpinspect-buffer-index-imports buffer (phpinspect-bmap-imports map))
           (phpinspect-buffer-index-declarations buffer (phpinspect-bmap-declarations map))
           (phpinspect-buffer-index-namespaces buffer (phpinspect-bmap-namespaces map))
@@ -497,6 +560,10 @@ continuing execution."
   (phpinspect-buffer-parse buffer)
   (phpinspect-buffer-map buffer))
 
+(define-inline phpinspect--atom-regexp ()
+  "A regular expression that matches (sequences of) atomic tokens."
+  (inline-quote "\\(\\$\\|->\\|::\\)?[^][)(}{[:blank:]\n;'\"]+"))
+
 (cl-defmethod phpinspect-buffer-register-edit
   ((buffer phpinspect-buffer) (start integer) (end integer) (pre-change-length integer))
   "Mark a region of the buffer as edited."
@@ -506,7 +573,7 @@ continuing execution."
   ;; they grow or shrink, so their full regions need to be marked for a reparse).
   (save-excursion
     (goto-char start)
-    (when (looking-back "\\(\\$\\|->\\|::\\)?[^][)(}{[:blank:]\n;'\"]+" nil t)
+    (when (looking-back (phpinspect--atom-regexp) nil t)
       (setq start (- start (length (match-string 0))))
       (setq pre-change-length (+ pre-change-length (length (match-string 0))))))
 
@@ -554,4 +621,12 @@ use."
   (when phpinspect-current-buffer
     (phpinspect-buffer-register-edit phpinspect-current-buffer start end pre-change-length)))
 
+(define-inline phpinspect-with-current-buffer (buffer &rest body)
+  (declare (indent 1))
+  (inline-letevals (buffer)
+    (push 'progn body)
+    (inline-quote
+     (with-current-buffer (phpinspect-buffer-buffer ,buffer)
+       ,body))))
+
 (provide 'phpinspect-buffer)
