@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.2.0
+;; Version: 2.0.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -33,6 +33,16 @@
 (require 'phpinspect-util)
 (require 'phpinspect-type)
 
+(defcustom phpinspect-imports-remove-unused nil
+  "Set to `t' to automatically remove unused imports.
+
+A value of `t' makes `phpinspect-fix-imports' automatically
+remove imports for unused types.  This is an experimental feature
+that may sometimes fail in identifying use of an import,
+resulting in an unjust removal."
+  :type 'boolean
+  :group 'phpinspect)
+
 (defun phpinspect-insert-at-point (point data)
   (save-excursion
     (goto-char point)
@@ -103,7 +113,7 @@ buffer position to insert the use statement at."
           ((> (length fqns) 1)
            (phpinspect-add-use (completing-read "Class: " (phpinspect-names-to-alist fqns))
                                buffer namespace-token))
-          (t (phpinspect-message "No import found for type %s" typename)))))
+          (t (phpinspect-message "No import found for type %s" (phpinspect-name-string typename))))))
 
 (defun phpinspect-namespace-part-of-typename (typename)
   (string-trim-right typename "\\\\?[^\\]+"))
@@ -357,8 +367,9 @@ that there are import (\"use\") statements for them."
         (phpinspect-add-use-statements-for-missing-types
          used-types buffer imports project (phpinspect-buffer-root-meta buffer))
 
-        (phpinspect-remove-unneeded-use-statements
-         used-types buffer imports (phpinspect-buffer-root-meta buffer))
+        (when phpinspect-imports-remove-unused
+	  (phpinspect-remove-unneeded-use-statements
+           used-types buffer imports (phpinspect-buffer-root-meta buffer)))
 
         (phpinspect-format-use-statements
          buffer (phpinspect-find-first-use (phpinspect-buffer-root-meta buffer)))
@@ -375,16 +386,12 @@ that there are import (\"use\") statements for them."
             (unless token-meta
               (error "Unable to find token for namespace %s" namespace-name))
 
-            (phpinspect-add-use-statements-for-missing-types
-             used-types buffer (append imports namespace-imports) project token-meta)
+	    (when phpinspect-imports-remove-unused
+              (phpinspect-add-use-statements-for-missing-types
+               used-types buffer (append imports namespace-imports) project token-meta))
 
-            (with-current-buffer (get-buffer-create "attempt-before-namespace-removal-code")
-              (erase-buffer)
-              (insert (with-current-buffer (phpinspect-buffer-buffer buffer) (buffer-string))))
-
-
-            (phpinspect-remove-unneeded-use-statements
-             used-types buffer (append imports namespace-imports) token-meta)
+            ;; (phpinspect-remove-unneeded-use-statements
+            ;;  used-types buffer (append imports namespace-imports) token-meta)
 
             (let ((parent (phpinspect-meta-find-parent-matching-token
                            token-meta #'phpinspect-namespace-or-root-p)))
