@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.2.0
+;; Version: 2.0.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -89,6 +89,8 @@ of TYPE, if available."
                       type-resolver current add-used-types comment-before))
 
                (when (setq return-type (seq-find #'phpinspect-word-p declaration))
+                 (when add-used-types
+                   (funcall add-used-types (list (cadr return-type))))
                  (setq return-type (funcall type-resolver
                                             (phpinspect--make-type :name (cadr return-type)))))
 
@@ -136,7 +138,7 @@ If ADD-USED-TYPES is set, it must be a function and will be
 called with a list of the types that are used within the
 function (think \"new\" statements, return types etc.)."
   (phpinspect--log "Indexing function")
-  (let* ((php-func (cadr scope))
+  (let* ((php-func (seq-find #'phpinspect-function-p scope))
          (declaration (cadr php-func))
          name type arguments throws used-types)
 
@@ -144,9 +146,10 @@ function (think \"new\" statements, return types etc.)."
                 (phpinspect--index-function-declaration
                  declaration type-resolver add-used-types comment-before))
 
-    ;; FIXME: Anonymous functions should not be indexed! (or if they are, they
+    ;; Note: Anonymous functions should not be indexed! (or if they are, they
     ;; should at least not be visible from various UIs unless assigned to a
-    ;; variable as a closure).
+    ;; variable as a closure). It is up to the caller of this function to
+    ;; determine the right behavior.
     (unless name (setq name "anonymous"))
 
     (phpinspect--log "Checking function return annotations")
@@ -157,7 +160,8 @@ function (think \"new\" statements, return types etc.)."
                  (cadadr (seq-find #'phpinspect-return-annotation-p comment-before))))
       (if type
           (phpinspect--apply-annotation-type return-annotation-type type type-resolver)
-        (setq type (funcall type-resolver (phpinspect--make-type :name return-annotation-type)))))
+        (when (stringp return-annotation-type)
+          (setq type (funcall type-resolver (phpinspect--make-type :name return-annotation-type))))))
 
     (when-let ((throw-annotations (seq-filter #'phpinspect-throws-annotation-p comment-before)))
       (dolist (tr throw-annotations)
@@ -170,7 +174,6 @@ function (think \"new\" statements, return types etc.)."
       (setq used-types (nconc used-types
                               (phpinspect--find-used-types-in-tokens
                                `(,(seq-find #'phpinspect-block-p php-func)))))
-      (when type (push (phpinspect--type-bare-name type) used-types))
       (funcall add-used-types used-types))
 
     (phpinspect--log "Creating function object")
@@ -179,7 +182,7 @@ function (think \"new\" statements, return types etc.)."
      :token php-func
      :name (concat (if namespace (concat namespace "\\") "") name)
      :throws throws
-     :return-type (or type phpinspect--null-type)
+     :return-type type
      :arguments arguments)))
 
 (define-inline phpinspect--safe-cadr (list)
@@ -327,6 +330,7 @@ SCOPE should be a scope token (`phpinspect-scope-p')."
   "Create an alist with relevant attributes of a parsed class."
   (phpinspect--log "INDEXING CLASS")
   (let ((methods)
+        (trait-config)
         (static-methods)
         (static-variables)
         (variables)
@@ -407,16 +411,15 @@ SCOPE should be a scope token (`phpinspect-scope-p')."
                                                           add-used-types)
                    methods))
             ((phpinspect-doc-block-p token)
-             (phpinspect--log "setting comment-before %s" token)
              (setq comment-before token))
 
             ;; Prevent comments from sticking around too long
             ((and (phpinspect-use-p token) (phpinspect-word-p (cadr token)))
-             ;; FIXME: Actually implement indexation of trait usage. This just
-             ;; marks the type as used for now.
-             (push (cadadr token) used-types))
+             ;; Trait use statement
+             (setq trait-config
+                   (nconc trait-config
+                          (phpinspect--index-trait-use token type-resolver add-used-types))))
             (t
-             (phpinspect--log "Unsetting comment-before")
              (setq comment-before nil))))
 
     ;; Dirty hack that assumes the constructor argument names to be the same as the object
@@ -433,7 +436,7 @@ SCOPE should be a scope token (`phpinspect-scope-p')."
                                         constructor-sym))
                                   methods)))
       (when constructor
-        (phpinspect--log "Constructor was found")
+        (phpinspect--log "Constructor was found for %s" class-name)
         (dolist (variable variables)
           (when (not (phpinspect--variable-type variable))
             (phpinspect--log "Looking for variable type in constructor arguments (%s)"
@@ -456,6 +459,7 @@ SCOPE should be a scope token (`phpinspect-scope-p')."
     `(,class-name .
                   (phpinspect--indexed-class
                    (complete . ,(not (phpinspect-incomplete-class-p class)))
+                   (trait-config . ,trait-config)
                    (class-name . ,class-name)
                    (declaration . ,(seq-find #'phpinspect-declaration-p class))
                    (location . ,(funcall location-resolver class))
@@ -558,13 +562,13 @@ NAMESPACE will be assumed the root namespace if not provided"
              (setq function (phpinspect--index-function-from-scope
                              type-resolver `(:public ,token) comment-before add-used-types
                              namespace))
-             (unless (phpinspect--function-anonyous-p function)
+             (unless (phpinspect--function-anonymous-p function)
              (push function functions)))
             ((phpinspect-block-or-list-p token)
              (dolist (fn (phpinspect--index-functions-in-tokens
                           (cdr token) type-resolver-factory imports namespace
                           add-used-types type-resolver))
-               (unless (phpinspect--function-anonyous-p fn)
+               (unless (phpinspect--function-anonymous-p fn)
                  (push fn functions))))))
 
     functions))
@@ -705,5 +709,67 @@ Returns a list of type name strings."
   "Index a PHP file for classes and the methods they have"
   (phpinspect--index-tokens (phpinspect-parse-current-buffer)))
 
+
+(defun phpinspect--index-trait-use (token type-resolver add-used-types)
+  (cl-assert (phpinspect-use-p token))
+  (setq token (cdr (seq-filter #'phpinspect-not-comment-p token)))
+
+  (let ((block? (car (last token)))
+        used-types config)
+    (when (phpinspect-end-of-statement-p block?)
+      (setq token (butlast token)))
+
+    (setq token (seq-filter #'phpinspect-not-comma-p token))
+
+    (dolist (word token)
+      (when (phpinspect-word-p word)
+        (push (cadr word) used-types)
+        (push `(,(funcall type-resolver (phpinspect--make-type :name (cadr word))))
+              config)))
+
+    (when (phpinspect-block-p block?)
+      (setq block? (cdr (seq-filter #'phpinspect-not-comment-p block?)))
+
+      (while block?
+        (cond ((phpinspect-comma-p (car block?))
+               (pop block?))
+              ;; Override
+              ((phpinspect--match-sequence block?
+                 :f #'phpinspect-word-p
+                 :f #'phpinspect-static-attrib-p
+                 :m '(:word "insteadof")
+                 :f #'phpinspect-word-p
+                 :rest *)
+               (let* ((type (funcall type-resolver (phpinspect--make-type :name (cadr (nth 3 block?)))))
+                      (t-config (assoc type config #'phpinspect--type=)))
+                 (when t-config
+                   (push `(override ,(cadadr (cadr block?))
+                                    ,(funcall type-resolver
+                                              (phpinspect--make-type :name (cadar block?))))
+                         (cdr t-config))))
+
+               (setq block? (nthcdr 4 block?)))
+
+              ;; alias
+              ((phpinspect--match-sequence block?
+                 :f #'phpinspect-word-p
+                 :f #'phpinspect-static-attrib-p
+                 :m '(:word "as")
+                 :f #'phpinspect-word-p
+                 :rest *)
+               (let* ((type (funcall type-resolver (phpinspect--make-type :name (cadar block?))))
+                      (t-config (assoc type config #'phpinspect--type=)))
+
+                 (when t-config
+                   (push `(alias ,(cadadr (cadr block?)) ,(cadr (nth 3 block?)))
+                         (cdr t-config))))
+
+               (setq block? (nthcdr 4 block?)))
+              (t (pop block?)))))
+
+    (when add-used-types
+      (funcall add-used-types used-types))
+    config))
+
 (provide 'phpinspect-index)
 ;;; phpinspect-index.el ends here
