@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.2.0
+;; Version: 2.0.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -40,14 +40,16 @@
      (progn
        (substring ,string 0 (- (length ,string) 1))))))
 
-(defsubst phpinspect-munch-token-without-attribs (string token-keyword)
+(define-inline phpinspect-munch-token-without-attribs (string token-keyword)
   "Return a token of type TOKEN-KEYWORD with STRING as value.
 If STRING has text properties, they are stripped."
-  (let ((value (copy-sequence string))
-        (length (length string)))
-    (forward-char length)
-    (set-text-properties 0 length nil value)
-    (list token-keyword value)))
+  (inline-letevals (string token-keyword)
+    (inline-quote
+     (let ((value (copy-sequence ,string))
+           (length (length ,string)))
+       (forward-char length)
+       (set-text-properties 0 length nil value)
+       (list ,token-keyword value)))))
 
 (eval-and-compile
   (defun phpinspect-handler-func-name (handler-name)
@@ -599,9 +601,15 @@ nature like argument lists"
 
 (phpinspect-defparser use
   :tree-keyword "use"
-  :handlers '(word tag block-without-scopes terminator)
+  :handlers '(comment word tag block-without-scopes comma terminator)
   :delimiter-predicate #'phpinspect-end-of-use-p)
 
+(phpinspect-defparser use-trait
+  :tree-keyword "use-trait"
+  :handlers '(comment word tag block-without-scopes comma terminator)
+  :delimiter-predicate #'phpinspect-end-of-use-p)
+
+
 (phpinspect-defhandler use-keyword (start-token max-point)
   "Handler for the use keyword and tokens that might follow to give it meaning"
   ((regexp . (concat "use" (phpinspect--word-end-regex))))
@@ -609,6 +617,14 @@ nature like argument lists"
   (forward-char (length start-token))
   (phpinspect--parse-use (current-buffer) max-point))
 
+(phpinspect-defhandler use-trait-keyword (start-token max-point)
+  "Handler for the use keyword within the body of a class."
+  ((regexp . (concat "use" (phpinspect--word-end-regex))))
+  (setq start-token (phpinspect--strip-word-end-space start-token))
+  (forward-char (length start-token))
+  (phpinspect--parse-use-trait (current-buffer) max-point))
+
+
 (phpinspect-defhandler attribute-reference (start-token &rest _ignored)
   "Handler for references to object attributes, or static class attributes."
   ((regexp . "->\\|::"))
@@ -694,13 +710,12 @@ static keywords with the same meaning as in a class block."
   :tree-keyword "block"
   :handlers '(array tag equals list comma string-concatenator attribute-reference
                     class-variable assignment-operator whitespace scope-keyword
-                    static-keyword const-keyword use-keyword function-keyword
+                    static-keyword const-keyword use-trait-keyword function-keyword
                     word terminator here-doc string comment block))
 
 (phpinspect-defhandler class-block (start-token max-point)
   "Handler for code blocks that cannot contain classes"
   ((regexp . "{"))
-  (forward-char (length start-token))
   (let* ((complete-block nil)
          (continue-condition (lambda ()
                                (not (and (char-equal (char-after) ?})
@@ -774,23 +789,13 @@ Returns the consumed text string without face properties."
   :handlers '(comment word list terminator tag)
   :delimiter-predicate #'phpinspect-end-of-token-p)
 
-;; TODO: Look into using different names for function and class :declaration tokens. They
-;; don't necessarily require the same handlers to parse.
-(define-inline phpinspect-parse-declaration (buffer max-point &optional continue-condition root)
-  (inline-quote
-     (let ((result (phpinspect--parse-declaration ,buffer ,max-point nil ,continue-condition ,root)))
-       (if (phpinspect-terminator-p (car (last result)))
-           (butlast result)
-         result))))
-
 (phpinspect-defhandler function-keyword (start-token max-point)
   "Handler for the function keyword and tokens that follow to give it meaning"
   ((regexp . (concat "function" (phpinspect--word-end-regex))))
   (setq start-token (phpinspect--strip-word-end-space start-token))
   (let* ((continue-condition (lambda () (not (or (char-equal (char-after) ?{)
-                                                 (char-equal (char-after) ?})))))
-         (declaration (phpinspect-parse-declaration (current-buffer) max-point continue-condition 'root)))
-
+                                                           (char-equal (char-after) ?})))))
+         (declaration (phpinspect--parse-declaration (current-buffer) max-point nil continue-condition 'root)))
     (if (phpinspect-end-of-token-p (car (last declaration)))
         ;; Abstract function?
         (list :function declaration)
@@ -888,9 +893,10 @@ the properties of the class"
   ((regexp . (concat "\\(abstract\\|final\\|class\\|interface\\|trait\\|enum\\)"
                      (phpinspect--word-end-regex))))
   (setq start-token (phpinspect--strip-word-end-space start-token))
-  `(:class ,(phpinspect-parse-declaration
+  `(:class ,(phpinspect--parse-declaration
                 (current-buffer)
                 max-point
+                nil
                 (lambda () (not (char-equal (char-after) ?{)))
                 'root)
            ,@(cdr (phpinspect--parse-class-body (current-buffer) max-point nil))))
