@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.2.0
+;; Version: 2.0.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -217,10 +217,10 @@ accompanied by all of its enclosing tokens."
 
     resolvecontext)))
 
-(defun phpinspect-rctx-get-or-create-cached-project-class (rctx class-fqn &optional no-enqueue)
+(defun phpinspect-rctx-get-typedef (rctx class-fqn &optional no-enqueue)
   (cl-assert (phpinspect--resolvecontext-p rctx))
   (let ((project (phpinspect--resolvecontext-project rctx)))
-    (phpinspect-project-get-class-extra-or-create project class-fqn no-enqueue)))
+    (phpinspect-project-get-typedef-extra-or-create project class-fqn no-enqueue)))
 
 (defun phpinspect-rctx-get-function-return-type (rctx function-name)
   (cl-assert (phpinspect--resolvecontext-p rctx))
