@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.2.0
+;; Version: 2.0.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -51,6 +51,9 @@ Type can be any of the token types returned by
 (defun phpinspect-comma-p (token)
   (phpinspect-token-type-p token :comma))
 
+(defun phpinspect-not-comma-p (token)
+  (not (phpinspect-comma-p token)))
+
 (defsubst phpinspect-terminator-p (token)
   (phpinspect-token-type-p token :terminator))
 
@@ -72,7 +75,7 @@ Type can be any of the token types returned by
 
 (defun phpinspect-end-of-use-p (token)
   (or (phpinspect-block-p token)
-      (phpinspect-end-of-token-p token)))
+      (phpinspect-terminator-p token)))
 
 (defun phpinspect-static-p (token)
   (phpinspect-token-type-p token :static))
@@ -243,8 +246,16 @@ Type can be any of the token types returned by
   (or (phpinspect-namespace-p object)
       (phpinspect-root-p object)))
 
+(define-inline phpinspect-use-import-p (token)
+  (inline-quote (phpinspect-token-type-p ,token :use)))
+
+(define-inline phpinspect-use-trait-p (token)
+  (inline-quote (phpinspect-token-type-p ,token :use-trait)))
+
 (define-inline phpinspect-use-p (object)
-  (inline-quote (phpinspect-token-type-p ,object :use)))
+  (inline-quote
+   (or (phpinspect-use-import-p ,object)
+       (phpinspect-use-trait-p ,object))))
 
 (defun phpinspect-comment-p (token)
   (or (phpinspect-token-type-p token :comment)
