@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.2.0
+;; Version: 2.0.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -47,6 +47,11 @@
             "When the type is a collection, this attribute is set to the type
 that the collection is expected to contain")
   (-bare-name-sym-slot nil)
+  (-display-name-slot nil)
+  (category nil
+            :documentation
+            "A symbol declaring whether this type is a class,
+ trait, abstract-class or interface.")
   (fully-qualified nil
                    :type bool
                    :documentation
@@ -81,6 +86,7 @@ that the collection is expected to contain")
 (defvar phpinspect--self-type (phpinspect--make-type :name "\\self" :fully-qualified t))
 (defvar phpinspect--this-type (phpinspect--make-type :name "\\this" :fully-qualified t))
 (defvar phpinspect--null-type (phpinspect--make-type :name "\\null" :fully-qualified t))
+(defvar phpinspect--unknown-type (phpinspect--make-type :name "unknown-type" :fully-qualified t))
 
 (defun phpinspect-define-standard-types ()
   (setq phpinspect-native-types
@@ -103,11 +109,10 @@ See https://wiki.php.net/rfc/static_return_type ."
   (or (phpinspect--type= type phpinspect--static-type)
       (phpinspect--type= type phpinspect--this-type)))
 
-(cl-defmethod phpinspect--resolve-late-static-binding
-  ((type phpinspect--type)
-   (class-type phpinspect--type))
+(defun phpinspect--resolve-late-static-binding (type origin-type)
+  "Resolve TYPE late static binding using ORIGIN-TYPE."
   (if (phpinspect--type-does-late-static-binding type)
-      class-type
+      origin-type
     type))
 
 (defsubst phpinspect--type-is-native (type)
@@ -140,9 +145,25 @@ See https://wiki.php.net/rfc/static_return_type ."
      (with-memoization (phpinspect--type--bare-name-sym-slot ,type)
        (phpinspect-intern-name (phpinspect--type-bare-name ,type))))))
 
-(cl-defmethod phpinspect--type= ((type1 phpinspect--type) (type2 phpinspect--type))
+(defun phpinspect--type= (type1 type2)
   (eq (phpinspect--type-name-symbol type1) (phpinspect--type-name-symbol type2)))
 
+(defun phpinspect--types-uniq (types)
+  "Optimized seq-uniq for types."
+  (let* (table
+         (filtered (cons nil nil))
+         (filtered-rear filtered))
+    (dolist (type types)
+      (let ((name (phpinspect--type-name-symbol type)))
+        (unless (memq name table)
+          (setq filtered-rear (setcdr filtered-rear (cons type nil)))
+          (push name table))))
+
+    (cdr filtered)))
+
+
+
+
 (defun phpinspect--resolve-type-name (types namespace type)
   "Get the FQN for TYPE, using TYPES and NAMESPACE as context.
 
@@ -232,13 +253,27 @@ NAMESPACE may be nil, or a string with a namespace FQN."
 (cl-defmethod phpinspect--display-format-type-name ((name string))
   (propertize (phpinspect--format-type-name name) 'face 'font-lock-type-face))
 
+(define-inline phpinspect--type-format-display-name (type)
+  (inline-letevals (type)
+    (inline-quote
+     (if (phpinspect--type-fully-qualified ,type)
+         ;; Save display name when name is fully qualified, as it won't change
+         ;; again.
+         (with-memoization (phpinspect--type--display-name-slot ,type)
+           (phpinspect--display-format-type-name (phpinspect--type-name ,type)))
+       (phpinspect--display-format-type-name (phpinspect--type-name ,type))))))
+
 (cl-defmethod phpinspect--display-format-type-name ((type phpinspect--type))
-  (let ((self (phpinspect--format-type-name type)))
-    (propertize
-     (if (phpinspect--type-contains type)
-        (concat self "<" (phpinspect--format-type-name (phpinspect--type-contains type)) ">")
-       self)
-     'face 'font-lock-type-face)))
+  (let ((self (phpinspect--type-format-display-name type)))
+    (if (phpinspect--type-contains type)
+        (concat self "<" (phpinspect--format-type-name (phpinspect--type-format-display-name type)) ">")
+      self)))
+
+(cl-defmethod phpinspect--display-format-type-name (type)
+  (cl-assert (not type))
+
+  (phpinspect--display-format-type-name "unknown-type"))
+
 
 (cl-defstruct (phpinspect--function (:constructor phpinspect--make-function-generated)
                                     (:copier phpinspect--copy-function))
@@ -280,7 +315,7 @@ return type of the function."))
 (defun phpinspect--function-argument-type (fn argument-name)
   (alist-get argument-name (phpinspect--function-arguments fn) nil nil #'string=))
 
-(defun phpinspect--function-anonyous-p (fn)
+(defun phpinspect--function-anonymous-p (fn)
   (eq (phpinspect-intern-name "anonymous") (phpinspect--function-name-symbol fn)))
 
 (defmacro phpinspect--make-function (&rest property-list)
@@ -364,7 +399,7 @@ mutability of the variable")
 
 (defun phpinspect--index-class-declaration (decl type-resolver)
   ;; Find out what the class extends or implements
-  (let (encountered-extends encountered-implements encountered-class
+  (let (keyword encountered-extends encountered-implements encountered-class
         class-name extends implements used-types)
     (dolist (word decl)
       (if (phpinspect-word-p word)
@@ -379,7 +414,8 @@ mutability of the variable")
                   (eval-when-compile
                     (concat "^" (phpinspect--class-keyword-handler-regexp) "?$"))
                   (cadr word))
-                 (setq encountered-class t))
+                 (setq keyword word
+                       encountered-class t))
                 (t
                  (phpinspect--log "Calling Resolver from index-class on %s" (cadr word))
                  (cond (encountered-extends
@@ -393,7 +429,15 @@ mutability of the variable")
                               implements)
                         (push (cadr word) used-types))
                        (encountered-class
-                        (setq class-name (funcall type-resolver (phpinspect--make-type :name (cadr word)))
+                        (setq class-name
+                              (funcall type-resolver (phpinspect--make-type
+                                                      :category (pcase (cadr keyword)
+                                                                  ("class" 'class)
+                                                                  ("trait" 'trait)
+                                                                  ("interface" 'interface)
+                                                                  ("enum" 'enum)
+                                                                  (_ 'class))
+                                                      :name (cadr word)))
                               encountered-class nil)))))))
 
     (list class-name extends implements used-types)))
