@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 1.2.0
+;; Version: 2.0.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -33,6 +33,9 @@ PHP code. Used to optimize string comparison. See also `phpinspect-intern-name'"
 (define-inline phpinspect-name-string (name)
   (inline-quote (cdr ,name)))
 
+(define-inline phpinspect-name-p (name)
+  (inline-quote (eq 'phpinspect-name (car ,name))))
+
 (defvar phpinspect-project-root-file-list
   '("composer.json" "composer.lock" ".git" ".svn" ".hg")
   "List of files that could indicate a project root directory.")
