@@ -132,14 +132,17 @@
       (should (phpinspect-function-p (phpinspect-meta-token (phpinspect-meta-parent (phpinspect-meta-parent bello)))))
       (should (phpinspect-function-p (phpinspect-meta-token (phpinspect-meta-parent (phpinspect-meta-parent bello1)))))
 
-      (let ((function (phpinspect-meta-token (phpinspect-meta-parent (phpinspect-meta-parent bello1)))))
-        ;; There's 5 nested tokens in the function. It is not a valid function
-        ;; token, a correct function would only contain a declaration and a
-        ;; block. But the incremental parser doesn't complain.
-        (should (= 5 (length function)))
+      (let* ((function-meta (phpinspect-meta-parent (phpinspect-meta-parent bello1)))
+             (function (phpinspect-meta-token function-meta)))
+        ;; The uncomplete function body was absorbed by the declaration up until
+        ;; the first semicolon.
+        (should (= 2 (length function)))
         (should (phpinspect-declaration-p (cadr function)))
         (should (member '(:word "Bello") (cadr function)))
-        (should (member '(:word "echo") (cadr function))))
+        (should (member '(:word "echo") (cadr function)))
+
+        ;; rest of tokens is absorbed by parent.
+        (should (equal `(:word "if")  (phpinspect-meta-token (phpinspect-meta-find-right-sibling function-meta)))))
 
       (phpinspect-document-apply-edit document 24 25 1 "{")
       (should (string= "<?php function Bello() { echo 'Hello World!'; if ($name) { echo 'Hello ' . $name . '!';} }"
@@ -424,11 +427,11 @@ use CCC;
     (phpinspect-buffer-index-namespaces buffer namespaces)
     (phpinspect-buffer-index-classes buffer classes)
 
-    (should (phpinspect-project-get-class (phpinspect-buffer-project buffer) (phpinspect--make-type :name "\\TestNamespace\\TestClass")))
+    (should (phpinspect-project-get-typedef (phpinspect-buffer-project buffer) (phpinspect--make-type :name "\\TestNamespace\\TestClass")))
 
-    (should (= 2 (hash-table-count (phpinspect-project-class-index (phpinspect-buffer-project buffer)))))
-    (should (= 1 (length (phpinspect--class-extended-classes
-                          (phpinspect-project-get-class
+    (should (= 2 (hash-table-count (phpinspect-project-typedef-index (phpinspect-buffer-project buffer)))))
+    (should (= 1 (length (phpi-typedef-subscribed-to-types
+                          (phpinspect-project-get-typedef
                            (phpinspect-buffer-project buffer)
                            (phpinspect--make-type :name "\\TestNamespace\\TestClass"))))))
 
@@ -451,12 +454,12 @@ use CCC;
 
       (phpinspect-buffer-index-declarations buffer new-declarations)
       (phpinspect-buffer-index-classes buffer new-classes)
-      (should (phpinspect-project-get-class
+      (should (phpinspect-project-get-typedef
                (phpinspect-buffer-project buffer)
                (phpinspect--make-type :name "\\TestNamespace\\TestClass")))
 
-      (should (= 0 (length (phpinspect--class-extended-classes
-                          (phpinspect-project-get-class
+      (should (= 0 (length (phpi-typedef-subscribed-to-types
+                          (phpinspect-project-get-typedef
                            (phpinspect-buffer-project buffer)
                            (phpinspect--make-type :name "\\TestNamespace\\TestClass")))))))
 
@@ -465,11 +468,11 @@ use CCC;
       (setf (phpinspect-bmap--root-meta (phpinspect-buffer-map buffer)) new-root)
       (phpinspect-buffer-index-classes buffer new-classes)
 
-      (should-not (phpinspect-project-get-class
+      (should-not (phpinspect-project-get-typedef
                    (phpinspect-buffer-project buffer)
                    (phpinspect--make-type :name "\\TestNamespace\\TestClass")))
 
-      (should (= 1 (hash-table-count (phpinspect-project-class-index (phpinspect-buffer-project buffer))))))))
+      (should (= 1 (hash-table-count (phpinspect-project-typedef-index (phpinspect-buffer-project buffer))))))))
 
 (ert-deftest phpinspect-buffer-index-functions ()
   (with-temp-buffer
@@ -486,23 +489,23 @@ class TestClass
       (phpinspect-buffer-update-project-index buffer)
 
 
-      (should (phpinspect-project-get-class
+      (should (phpinspect-project-get-typedef
                (phpinspect-buffer-project buffer)
                (phpinspect--make-type :name "\\NS\\TestClass")))
 
-      (should (= 1 (hash-table-count (phpinspect--class-methods
-                                      (phpinspect-project-get-class
-                                       (phpinspect-buffer-project buffer)
-                                       (phpinspect--make-type :name "\\NS\\TestClass"))))))
+      (should (= 1 (length (phpi-typedef-get-methods
+                            (phpinspect-project-get-typedef
+                             (phpinspect-buffer-project buffer)
+                             (phpinspect--make-type :name "\\NS\\TestClass"))))))
 
       (setf (phpinspect-buffer-map buffer) (phpinspect-make-bmap :-root-meta (phpinspect-make-meta nil 1 400 "" 'root)))
 
       (phpinspect-buffer-index-functions buffer (phpinspect-make-splayt))
 
-      (should (= 0 (hash-table-count (phpinspect--class-methods
-                                      (phpinspect-project-get-class
-                                       (phpinspect-buffer-project buffer)
-                                       (phpinspect--make-type :name "\\NS\\TestClass")))))))))
+      (should (= 0 (length (phpi-typedef-get-methods
+                            (phpinspect-project-get-typedef
+                             (phpinspect-buffer-project buffer)
+                             (phpinspect--make-type :name "\\NS\\TestClass")))))))))
 
 (ert-deftest phpinspect-buffer-index-class-variables ()
   (let ((buffer (phpinspect-make-buffer :-project (phpinspect--make-project :autoload (phpinspect-make-autoloader))))
@@ -542,25 +545,25 @@ class TestClass
 
     (phpinspect-buffer-index-class-variables buffer variables)
 
-    (should (phpinspect-project-get-class
+    (should (phpinspect-project-get-typedef
              (phpinspect-buffer-project buffer)
              (phpinspect--make-type :name "\\TestClass")))
 
-    (should (= 2 (length (phpinspect--class-variables
-                          (phpinspect-project-get-class
+    (should (= 1 (length (phpi-typedef-get-properties
+                          (phpinspect-project-get-typedef
                            (phpinspect-buffer-project buffer)
                            (phpinspect--make-type :name "\\TestClass"))))))
 
 
-    (should (= 1 (length (phpinspect--class-get-constants
-                          (phpinspect-project-get-class
+    (should (= 1 (length (phpi-typedef-get-constants
+                          (phpinspect-project-get-typedef
                            (phpinspect-buffer-project buffer)
                            (phpinspect--make-type :name "\\TestClass"))))))
 
     (should (phpinspect--type= (phpinspect--make-type :name "\\array")
-                              (phpinspect--variable-type
-                               (phpinspect--class-get-variable
-                                (phpinspect-project-get-class
+                              (phpi-var-type
+                               (phpi-typedef-get-property
+                                (phpinspect-project-get-typedef
                                  (phpinspect-buffer-project buffer)
                                  (phpinspect--make-type :name "\\TestClass"))
                                 "banana"))))))
@@ -616,19 +619,19 @@ class AccountStatisticsController {
 
       (phpinspect-buffer-update-project-index buffer)
 
-      (let ((class (phpinspect-project-get-class
+      (let ((class (phpinspect-project-get-typedef
                     (phpinspect-buffer-project buffer)
                     (phpinspect--make-type
                      :name "\\App\\Controller\\Api\\V1\\AccountStatisticsController"
                      :fully-qualified t))))
         (should class)
 
-        (let ((model (phpinspect--class-get-variable class "model"))
-              (priv-model (phpinspect--class-get-variable class "privModel"))
+        (let ((model (phpi-typedef-get-property class "model"))
+              (priv-model (phpi-typedef-get-property class "privModel"))
               ;; Static variables are stored with "$" prefix
-              (relation (phpinspect--class-get-variable class "$relation"))
-              (static-relation (phpinspect--class-get-variable class "$staticRelation"))
-              (relations (phpinspect--class-get-variable class "relations")))
+              (relation (phpi-typedef-get-property class "$relation"))
+              (static-relation (phpi-typedef-get-property class "$staticRelation"))
+              (relations (phpi-typedef-get-property class "relations")))
           (should model)
           (should priv-model)
           (should relation)
@@ -643,16 +646,167 @@ class AccountStatisticsController {
                              :fully-qualified t))
                 (array-type (phpinspect--make-type :name "\\array" :fully-qualified t)))
 
-            (should (phpinspect--variable-type model))
-            (should (phpinspect--type= model-type (phpinspect--variable-type model)))
-            (should (phpinspect--variable-type priv-model))
-            (should (phpinspect--type= model-type (phpinspect--variable-type priv-model)))
-            (should (phpinspect--variable-type relation))
-            (should (phpinspect--type= relation-type (phpinspect--variable-type relation)))
-            (should (phpinspect--variable-type static-relation))
-            (should (phpinspect--type= relation-type (phpinspect--variable-type static-relation)))
+            (should (phpi-var-type model))
+            (should (phpinspect--type= model-type (phpi-var-type model)))
+            (should (phpi-var-type priv-model))
+            (should (phpinspect--type= model-type (phpi-var-type priv-model)))
+            (should (phpi-var-type relation))
+            (should (phpinspect--type= relation-type (phpi-var-type relation)))
+            (should (phpi-var-type static-relation))
+            (should (phpinspect--type= relation-type (phpi-var-type static-relation)))
 
-            (should (phpinspect--type= array-type (phpinspect--variable-type relations)))
+            (should (phpinspect--type= array-type (phpi-var-type relations)))
             (should (phpinspect--type=
                      relation-type
-                     (phpinspect--type-contains (phpinspect--variable-type relations))))))))))
+                     (phpinspect--type-contains (phpi-var-type relations))))))))))
+
+
+(ert-deftest phpinspect-buffer-parse-incrementally-unfinished-variable-scope ()
+  (with-temp-buffer
+  (let* ((buffer (phpinspect-make-buffer
+                  :buffer (current-buffer)
+                  :-project (phpinspect--make-dummy-project))))
+
+
+    (setq-local phpinspect-current-buffer buffer)
+    (insert
+     "<?php
+
+namespace XXX;
+
+class AAA {
+
+    function bbb () {
+        $banana = 'aaa';
+    }
+}
+
+")
+
+    (add-hook 'after-change-functions #'phpinspect-after-change-function)
+    (phpinspect-buffer-parse buffer 'no-interrupt)
+
+    ;; move to after class brace
+    (goto-char 35)
+    (insert "\n public Foo ")
+    (phpinspect-buffer-parse buffer 'no-interrupt)
+    (phpinspect-buffer-update-project-index buffer)
+
+    ;; We're just confirming that the above code doesn't error
+    (should t))))
+
+(ert-deftest phpinspect-buffer-parse-incrementally-trait-config ()
+  (with-temp-buffer
+    (let* ((project (phpinspect--make-dummy-composer-project-with-code))
+           (buffer (phpinspect-make-buffer :-project project :buffer (current-buffer))))
+
+      (insert "<?php
+
+namespace Foo;
+
+class Bar {
+
+    function baz(): string {}
+}")
+
+      ;; Make sure trait typedef is loaded
+      (should (phpinspect-project-get-typedef-extra-or-create
+               project (phpinspect--make-type :name "\\App\\Foo") 'no-enqueue))
+
+      (setq-local phpinspect-current-buffer buffer)
+      (add-hook 'after-change-functions #'phpinspect-after-change-function)
+      (phpinspect-buffer-parse buffer 'no-interrupt)
+      (phpinspect-buffer-update-project-index buffer)
+
+      (let ((class (phpinspect-project-get-typedef
+                    project (phpinspect--make-type :name "\\Foo\\Bar"))))
+
+        (should class)
+        (should (= 1 (length (phpi-typedef-get-methods class))))
+
+        (goto-char 36)
+        (insert "use \\App\\Foo { \\App\\Foo::do as dooo }")
+
+        (phpinspect-buffer-parse buffer 'no-interrupt)
+        (phpinspect-buffer-update-project-index buffer)
+
+        (should (= 2 (length (phpi-typedef-get-methods class))))
+
+        (goto-char 36)
+        (kill-line)
+        (phpinspect-buffer-parse buffer 'no-interrupt)
+        (phpinspect-buffer-update-project-index buffer)
+
+        (should (= 1 (length (phpi-typedef-get-methods class))))))))
+
+
+(ert-deftest phpinspect-buffer-parse-incrementally-class-block-scope ()
+  (with-temp-buffer
+    (let* ((project (phpinspect--make-dummy-composer-project-with-code))
+           (buffer (phpinspect-make-buffer :-project project :buffer (current-buffer))))
+
+      (insert "<?php class A { public function A() {} }")
+
+      (setq-local phpinspect-current-buffer buffer)
+      (add-hook 'after-change-functions #'phpinspect-after-change-function)
+      (let ((expected `(:root
+			(:class
+			 (:declaration
+			  (:word "class")
+			  (:word "A"))
+			 (:block
+			  (:public ;; This test explicitly tests that the
+				   ;; "public" word doesn't lose its first
+				   ;; character after edits.
+			   (:function
+			    (:declaration
+			     (:word "function")
+			     (:word "A")
+			     (:list))
+			    (:block)))))))
+	    (result (phpinspect-buffer-parse buffer 'no-interrupt)))
+
+	(should result)
+	(should (equal expected result))
+
+	(goto-char 17)
+	(delete-char -1)
+	(setq result (phpinspect-buffer-parse buffer 'no-interrupt))
+	(should result)
+	(should (equal expected result))
+
+	(backward-char)
+	(insert " ")
+	(setq result (phpinspect-buffer-parse buffer 'no-interrupt))
+	(should result)
+	(should (equal expected result))))))
+
+(ert-deftest phpinspect-buffer-index-update-method-name ()
+    (with-temp-buffer
+      (let* ((project (phpinspect--make-project :autoload (phpinspect-make-autoloader)))
+	     (buffer (phpinspect-make-buffer :buffer (current-buffer) :-project project))
+	     (class-type (phpinspect--make-type :name "\\NS\\TestClass" :fully-qualified t)))
+      (insert "<?php
+namespace NS;
+
+class TestClass
+{
+    function testMe(): RelativeType {}
+}")
+      (setq-local phpinspect-current-buffer buffer)
+      (add-hook 'after-change-functions #'phpinspect-after-change-function)
+      (phpinspect-buffer-update-project-index buffer)
+
+      (goto-char 59)
+      (insert "thod")
+
+      (phpinspect-buffer-parse buffer 'no-interrupt)
+      (phpinspect-buffer-update-project-index buffer)
+
+      (let ((typedef (phpinspect-project-get-typedef project class-type)))
+	(should typedef)
+
+	(should (length= (phpi-typedef-get-methods typedef) 1 ))
+
+	(let ((method (phpi-typedef-get-method typedef "testMethod")))
+	  (should method))))))
